
/*
** Common constants for async_sleep_drv driver, MUST KEEP IN SYNC WITH
** berkeley_db.hrl!!
**
** Copyright (c) 2002, Scott Lystig Fritchie.  All rights reserved.
** See the file "LICENSE" at the top of the source distribution for
** full license terms.
**
*/

#define	BDB_DEBUG		0
#define	BDB_ENV_CREATE		1
#define	BDB_ENV_OPEN		2
#define	BDB_ENV_CLOSE		3
#define	BDB_ENV_REMOVE		4
#define	BDB_ENV_SET_RPC_SERVER	5
#define	BDB_TXN_BEGIN		10
#define	BDB_TXN_ABORT		11
#define	BDB_TXN_COMMIT		12
#define	BDB_TXN_SET_TIMEOUT	13
#define	BDB_DB_CREATE		100
#define	BDB_DB_OPEN		101
#define	BDB_DB_PUT		102
#define	BDB_DB_CLOSE		103
#define	BDB_DB_DEL		104
#define	BDB_DB_GET		105
#define	BDB_DB_SET_PAGESIZE	106
#define	BDB_DB_SET_CACHESIZE	107
#define	BDB_DB_SET_LORDER	108
#define	BDB_DB_SET_FLAGS	109
#define	BDB_DB_SET_BT_MINKEY	110
#define	BDB_DB_SET_H_FFACTOR	111
#define	BDB_DB_SET_H_NELEM	112
#define	BDB_DB_SET_RE_DELIM	113
#define	BDB_DB_SET_RE_LEN	114
#define	BDB_DB_SET_RE_PAD	115
#define	BDB_DB_SET_RE_SOURCE	116
#define	BDB_DB_UPGRADE		117
#define	BDB_DB_STAT		118
#define	BDB_DB_TRUNCATE		119
#define	BDB_DB_VERIFY		120
#define	BDB_DB_SYNC		121
#define	BDB_DB_REMOVE		122
#define	BDB_DB_CURSOR		123
#define	BDB_C_GET		124
#define	BDB_C_PUT		125
#define	BDB_C_DEL		126
#define	BDB_C_DUP		127
#define	BDB_DB_JOIN		128
#define	BDB_C_COUNT		129
#define	BDB_C_CLOSE		130

/* Sizing constants */

#define	BDB_MAX_TABLES		32 /* Max # open tables */
#define	BDB_MAX_TXNS		32 /* Max # pending transactions */
#define	BDB_MAX_CURSORS		32 /* Max # open cursors */
#define	BDB_NULL_TXN		-1 /* Used to specify the NULL transaction */

/* Other errors */

#define	BDB_ERR_CURSOROPEN	(-1600)

/*
** Other common and generally useful stuff.
*/

/*********************************************************************
 * ErlIOVec manipulation functions, stolen from efile_drv.c
 */

/* char *EV_CHAR_P(ErlIOVec *ev, int p, int q) */
#define EV_CHAR_P(ev, p, q) \
    (((char *)(ev)->iov[(q)].iov_base) + (p))

/* char *EV_UCHAR_P(ErlIOVec *ev, int p, int q) */
#define EV_UCHAR_P(ev, p, q) \
    (((unsigned char *)(ev)->iov[(q)].iov_base) + (p))

/* int EV_GET_CHAR(ErlIOVec *ev, char *p, int *pp, int *qp) */
#define EV_GET_CHAR(ev, p, pp, qp) \
    (*(pp)+1 <= (ev)->iov[*(qp)].iov_len \
     ? (*(p) = *EV_CHAR_P(ev, *(pp), *(qp)), \
        *(pp) = (*(pp)+1 < (ev)->iov[*(qp)].iov_len \
                 ? *(pp)+1 \
                 : ((*(qp))++, 0)), \
        !0) \
     : 0)

/* int EV_GET_UINT32(ErlIOVec *ev, Uint32 *p, int *pp, int *qp) */
#define EV_GET_UINT32(ev, p, pp, qp) \
    (*(pp)+4 <= (ev)->iov[*(qp)].iov_len \
     ? (*(p) = (*EV_UCHAR_P(ev, *(pp), *(qp)) << 24) \
                | (*EV_UCHAR_P(ev, *(pp)+1, *(qp)) << 16) \
                | (*EV_UCHAR_P(ev, *(pp)+2, *(qp)) << 8)\
                | *EV_UCHAR_P(ev, *(pp)+3, *(qp)), \
        *(pp) = (*(pp)+4 < (ev)->iov[*(qp)].iov_len \
                 ? *(pp)+4 \
                 : ((*(qp))++, 0)), \
        !0) \
     : 0)

/* char * EV_GETPOS(ErlIOVec *ev, int p, int q) */
#define EV_GETPOS(ev, p, q) \
    ((q) < (ev)->vsize \
    ? ((ev)->iov[(q)].iov_base + p) \
    : NULL)

/*********************************************************************
 * Term manipulation functions, stolen from inet_drv.c
 */

#define LOAD_ATOM(vec, i, atom) \
  (((vec)[(i)] = ERL_DRV_ATOM), \
  ((vec)[(i)+1] = (atom)), \
  (i+2))

#define LOAD_INT(vec, i, val) \
  (((vec)[(i)] = ERL_DRV_INT), \
  ((vec)[(i)+1] = (ErlDrvTermData)(val)), \
  (i+2))

#define LOAD_PORT(vec, i, port) \
  (((vec)[(i)] = ERL_DRV_PORT), \
  ((vec)[(i)+1] = (port)), \
  (i+2))

#define LOAD_PID(vec, i, pid) \
  (((vec)[(i)] = ERL_DRV_PID), \
  ((vec)[(i)+1] = (pid)), \
  (i+2))

#define LOAD_BINARY(vec, i, bin, offs, len) \
  (((vec)[(i)] = ERL_DRV_BINARY), \
  ((vec)[(i)+1] = (ErlDrvTermData)(bin)), \
  ((vec)[(i)+2] = (len)), \
  ((vec)[(i)+3] = (offs)), \
  (i+4))

#define LOAD_STRING(vec, i, str, len) \
  (((vec)[(i)] = ERL_DRV_STRING), \
  ((vec)[(i)+1] = (ErlDrvTermData)(str)), \
  ((vec)[(i)+2] = (len)), \
  (i+3))

#define LOAD_STRING_CONS(vec, i, str, len) \
  (((vec)[(i)] = ERL_DRV_STRING_CONS), \
  ((vec)[(i)+1] = (ErlDrvTermData)(str)), \
  ((vec)[(i)+2] = (len)), \
  (i+3))

#define LOAD_TUPLE(vec, i, size) \
  (((vec)[(i)] = ERL_DRV_TUPLE), \
  ((vec)[(i)+1] = (size)), \
  (i+2))

#define LOAD_LIST(vec, i, size) \
  (((vec)[(i)] = ERL_DRV_LIST), \
  ((vec)[(i)+1] = (size)), \
  (i+2))

#define get_int32(s) ((((unsigned char*) (s))[0] << 24) | \
                      (((unsigned char*) (s))[1] << 16)  | \
                      (((unsigned char*) (s))[2] << 8) | \
                      (((unsigned char*) (s))[3]))

