
%%% Copyright (c) 2002, Scott Lystig Fritchie.  All rights reserved.
%%% See the file "LICENSE" at the top of the source distribution for
%%% full license terms.
%%%

-define(DRV_NAME, "berkeley_db_drv").

%%%
%%% Driver<->emulator communication codes (xref with top of berkeley_db_drv.h)
%%%

-define(BDB_DEBUG,			0).
-define(BDB_ENV_CREATE,			1).
-define(BDB_ENV_OPEN,			2).
-define(BDB_ENV_CLOSE,			3).
-define(BDB_ENV_REMOVE,			4).
-define(BDB_ENV_SET_RPC_SERVER,		5).
-define(BDB_TXN_BEGIN,			10).
-define(BDB_TXN_ABORT,			11).
-define(BDB_TXN_COMMIT,			12).
-define(BDB_TXN_SET_TIMEOUT,		13).
-define(BDB_DB_CREATE,			100).
-define(BDB_DB_OPEN,			101).
-define(BDB_DB_PUT,			102).
-define(BDB_DB_CLOSE,			103).
-define(BDB_DB_DEL,			104).
-define(BDB_DB_GET,			105).
-define(BDB_DB_SET_PAGESIZE,		106).
-define(BDB_DB_SET_CACHESIZE,		107).
-define(BDB_DB_SET_LORDER,		108).
-define(BDB_DB_SET_FLAGS,		109).
-define(BDB_DB_SET_BT_MINKEY,		110).
-define(BDB_DB_SET_H_FFACTOR,		111).
-define(BDB_DB_SET_H_NELEM,		112).
-define(BDB_DB_SET_RE_DELIM,		113).
-define(BDB_DB_SET_RE_LEN,		114).
-define(BDB_DB_SET_RE_PAD,		115).
-define(BDB_DB_SET_RE_SOURCE,		116).
-define(BDB_DB_UPGRADE,			117).
-define(BDB_DB_STAT,			118).
-define(BDB_DB_TRUNCATE,		119).
-define(BDB_DB_VERIFY,			120).
-define(BDB_DB_SYNC,			121).
-define(BDB_DB_REMOVE,			122).
-define(BDB_DB_CURSOR,			123).
-define(BDB_C_GET,			124).
-define(BDB_C_PUT,			125).
-define(BDB_C_DEL,			126).
-define(BDB_C_DUP,			127).
-define(BDB_DB_JOIN,			128).
-define(BDB_C_COUNT,			129).
-define(BDB_C_CLOSE,			130).

%%%
%%% Sizing constants
%%%

-define(BDB_MAX_TABLES,                 32).
-define(BDB_MAX_TXNS,                   32).
-define(BDB_MAX_CURSORS,                32).
-define(BDB_NULL_TXN,                   -1).

%%%
%%% Other errors
%%%

-define(BDB_ERR_CURSOROPEN,             (-1600)).

%%%
%%% Begin constants from db.h
%%%

-define(DB_AFTER,                 1). %      /* c_put() */
-define(DB_APPEND,                2). %      /* put() */
-define(DB_BEFORE,                3). %      /* c_put() */
-define(DB_CACHED_COUNTS,         4). %      /* stat() */
-define(DB_CHECKPOINT,            5). %      /* log_put(), DB_LOGC->get() */
-define(DB_COMMIT,                6). %      /* log_put() (internal) */
-define(DB_CONSUME,               7). %      /* get() */
-define(DB_CONSUME_WAIT,          8). %      /* get() */
-define(DB_CURLSN,                9). %      /* log_put() */
-define(DB_CURRENT,              10). %      /* c_get(), c_put(), DB_LOGC->get() */
-define(DB_FAST_STAT,            11). %      /* stat() */
-define(DB_FIRST,                12). %      /* c_get(), DB_LOGC->get() */
-define(DB_GET_BOTH,             13). %      /* get(), c_get() */
-define(DB_GET_BOTHC,            14). %      /* c_get() (internal) */
-define(DB_GET_BOTH_RANGE,       15). %      /* get(), c_get() */
-define(DB_GET_RECNO,            16). %      /* c_get() */
-define(DB_JOIN_ITEM,            17). %      /* c_get(); do not do primary lookup */
-define(DB_KEYFIRST,             18). %      /* c_put() */
-define(DB_KEYLAST,              19). %      /* c_put() */
-define(DB_LAST,                 20). %      /* c_get(), DB_LOGC->get() */
-define(DB_NEXT,                 21). %      /* c_get(), DB_LOGC->get() */
-define(DB_NEXT_DUP,             22). %      /* c_get() */
-define(DB_NEXT_NODUP,           23). %      /* c_get() */
-define(DB_NODUPDATA,            24). %      /* put(), c_put() */
-define(DB_NOOVERWRITE,          25). %      /* put() */
-define(DB_NOSYNC,               26). %      /* close() */
-define(DB_POSITION,             27). %      /* c_dup() */
-define(DB_POSITIONI,            28). %      /* c_dup() (internal) */
-define(DB_PREV,                 29). %      /* c_get(), DB_LOGC->get() */
-define(DB_PREV_NODUP,           30). %      /* c_get(), DB_LOGC->get() */
-define(DB_RECORDCOUNT,          31). %      /* stat() */
-define(DB_SET,                  32). %      /* c_get(), DB_LOGC->get() */
-define(DB_SET_LOCK_TIMEOUT,     33). %      /* set_timout() */
-define(DB_SET_RANGE,            34). %      /* c_get() */
-define(DB_SET_RECNO,            35). %      /* get(), c_get() */
-define(DB_SET_TXN_NOW,          36). %      /* set_timout() (internal) */
-define(DB_SET_TXN_TIMEOUT,      37). %      /* set_timout() */
-define(DB_UPDATE_SECONDARY,     38). %      /* c_get(), c_del() (internal) */
-define(DB_WRITECURSOR,          39). %      /* cursor() */
-define(DB_WRITELOCK,            40). %      /* cursor() (internal) */

-define(DB_OPFLAGS_MASK, 0x000000ff). %      /* Mask for operations flags. */
-define(DB_FLUSH,        0x02000000). %      /* Flush data to disk. */
-define(DB_MULTIPLE,     0x04000000). %      /* Return multiple data values. */
-define(DB_MULTIPLE_KEY, 0x08000000). %      /* Return multiple data/key pairs. */
-define(DB_RMW,          0x10000000). %      /* Acquire write flag immediately. */

-define(DB_BTREE, 1).
-define(DB_HASH,  2).
-define(DB_RECNO, 3).
-define(DB_QUEUE, 4).

-define(DB_CREATE,             16#000001). %  /* Create file as necessary. */
-define(DB_CXX_NO_EXCEPTIONS,  16#000002). %  /* C++: return error values. */
-define(DB_FORCE,              16#000004). %  /* Force (anything). */
-define(DB_NOMMAP,             16#000008). %  /* Don't mmap underlying file. */
-define(DB_RDONLY,             16#000010). %  /* Read-only (O_RDONLY). */
-define(DB_RECOVER,            16#000020). %  /* Run normal recovery. */
-define(DB_THREAD,             16#000040). %  /* Applications are threaded. */
-define(DB_TXN_NOSYNC,         16#000080). %  /* Do not sync log on commit. */
-define(DB_USE_ENVIRON,        16#000100). %  /* Use the environment. */
-define(DB_USE_ENVIRON_ROOT,   16#000200). %  /* Use the environment if root. */
-define(DB_DIRTY_READ,       16#01000000). %  /* Dirty Read. */
-define(DB_CLIENT,             16#000400). %  /* Open for a client environment. */
-define(DB_XA_CREATE,          16#000400). %  /* Open in an XA environment. */
-define(DB_INIT_CDB,           16#000400). %  /* Concurrent Access Methods. */
-define(DB_INIT_LOCK,          16#000800). %  /* Initialize locking. */
-define(DB_INIT_LOG,           16#001000). %  /* Initialize logging. */
-define(DB_INIT_MPOOL,         16#002000). %  /* Initialize mpool. */
-define(DB_INIT_TXN,           16#004000). %  /* Initialize transactions. */
-define(DB_JOINENV,            16#008000). %  /* Initialize all subsystems present. */
-define(DB_LOCKDOWN,           16#010000). %  /* Lock memory into physical core. */
-define(DB_PRIVATE,            16#020000). %  /* DB_ENV is process local. */
-define(DB_RECOVER_FATAL,      16#040000). %  /* Run catastrophic recovery. */
-define(DB_SYSTEM_MEM,         16#080000). %  /* Use system-backed memory. */
-define(DB_EXCL,               16#000400). %  /* Exclusive open (O_EXCL). */
-define(DB_FCNTL_LOCKING,      16#000800). %  /* UNDOC: fcntl(2) locking. */
-define(DB_RDWRMASTER,         16#001000). %  /* UNDOC: allow subdb master open R/W */
-define(DB_TRUNCATE,           16#002000). %  /* Discard existing DB (O_TRUNC). */
-define(DB_EXTENT,             16#004000). %  /* UNDOC: dealing with an extent. */
-define(DB_APPLY_LOGREG,       16#008000). %  /* UNDOC: log_register in replicat. */
-define(DB_TXN_NOWAIT,         16#000400). %  /* Do not wait for locks in this TXN. */
-define(DB_TXN_SYNC,           16#000800). %  /* Always sync log on commit. */
-define(DB_CDB_ALLDB,          16#000400). %  /* Set CDB locking per environment. */
-define(DB_NOLOCKING,          16#001000). %  /* Set locking/mutex behavior. */
-define(DB_NOPANIC,            16#002000). %  /* Set panic state per DB_ENV. */
-define(DB_PANIC_ENVIRONMENT,  16#004000). %  /* Set panic state per environment. */
-define(DB_REGION_INIT,        16#008000). %  /* Set panic state per DB_ENV. */
-define(DB_YIELDCPU,           16#010000). %  /* Yield the CPU (a lot). */
-define(DB_UPGRADE,            16#000400). %  /* Upgrading. */
-define(DB_VERIFY,             16#000800). %  /* Verifying. */
-define(DB_ODDFILESIZE,        16#000400). %  /* Truncate file to N * pgsize. */
-define(DB_DUP,                16#000001). %  /* Btree, Hash: duplicate keys. */
-define(DB_DUPSORT,            16#000002). %  /* Btree, Hash: duplicate keys. */
-define(DB_RECNUM,             16#000004). %  /* Btree: record numbers. */
-define(DB_RENUMBER,           16#000008). %  /* Recno: renumber on insert/delete. */
-define(DB_REVSPLITOFF,        16#000010). %  /* Btree: turn off reverse splits. */
-define(DB_SNAPSHOT,           16#000020). %  /* Recno: snapshot the input. */
-define(DB_STAT_CLEAR,         16#000001). %  /* Clear stat after returning values. */
-define(DB_JOIN_NOSORT,        16#000001). %  /* Don't try to optimize join. */
-define(DB_AGGRESSIVE,         16#000001). %  /* Salvage whatever could be data.*/
-define(DB_NOORDERCHK,         16#000002). %  /* Skip sort order/hashing check. */
-define(DB_ORDERCHKONLY,       16#000004). %  /* Only perform the order check. */
-define(DB_PR_PAGE,            16#000008). %  /* Show page contents (-da). */
-define(DB_PR_RECOVERYTEST,    16#000010). %  /* Recovery test (-dr). */
-define(DB_SALVAGE,            16#000020). %  /* Salvage what looks like data. */
-define(DB_REP_PERMANENT,        16#0001). %  /* Important--app. may want to flush. */

-define(DB_DONOTINDEX,      -30999).%/* "Null" return from 2ndary callbk. */
-define(DB_INCOMPLETE,      -30998).%/* Sync didn't finish. */
-define(DB_KEYEMPTY,        -30997).%/* Key/data deleted or never created. */
-define(DB_KEYEXIST,        -30996).%/* The key/data pair already exists. */
-define(DB_LOCK_DEADLOCK,   -30995).%/* Deadlock. */
-define(DB_LOCK_NOTGRANTED, -30994).%/* Lock unavailable. */
-define(DB_NOSERVER,        -30993).%/* Server panic return. */
-define(DB_NOSERVER_HOME,   -30992).%/* Bad home sent to server. */
-define(DB_NOSERVER_ID,     -30991).%/* Bad ID sent to server. */
-define(DB_NOTFOUND,        -30990).%/* Key/data pair not found (EOF). */
-define(DB_OLD_VERSION,     -30989).%/* Out-of-date version. */
-define(DB_PAGE_NOTFOUND,   -30988).%/* Requested page not found. */
-define(DB_REP_DUPMASTER,   -30987).%/* There are two masters. */
-define(DB_REP_HOLDELECTION,-30986).%/* Time to hold an election. */
-define(DB_REP_NEWMASTER,   -30985).%/* We have learned of a new master. */
-define(DB_REP_NEWSITE,     -30984).%/* New site entered system. */
-define(DB_REP_OUTDATED,    -30983).%/* Site is too far behind master. */
-define(DB_REP_UNAVAIL,     -30982).%/* Site cannot currently be reached. */
-define(DB_RUNRECOVERY,     -30981).%/* Panic return. */
-define(DB_SECONDARY_BAD,   -30980).%/* Secondary index corrupt. */
-define(DB_VERIFY_BAD,      -30979).%/* Verify failed; bad format. */
